PhenGO_VERSION = "v0.1.0"
DEFAULT_WORM_PHENOTYPES_FILE = "data/worm/WS297_lethal_terms.tsv.gz"
DEFAULT_MOUSE_PHENOTYPES_FILE = "data/mouse/mouse_lethal_terms.txt.gz"
DEFAULT_FLY_FIELDS_FILE = "data/fly/FlyBase_Fields_2017.txt.gz"
DEFAULT_FLY_DRIVER_LINES_FILE = "data/fly/FlyBase_DriverLine_Fields_2025_08_05.txt.gz"